#include "CellLattice.h"
#include <stdio.h>

CellLattice::CellLattice() {
	n_cols = 0;
	n_rows = 0;

	matrix = NULL;

}

void CellLattice::Init(int max_rowlen, int num_cells) {
	n_cols = max_rowlen;
	n_rows = num_cells;

	// generate a matrix of pointers
	matrix = new long[n_rows*n_cols];

	// intially fill the matrix with NULL
	for(int i=0;i<n_rows*n_cols;i++) matrix[i] = NULL;

	isClone = false;
}

void CellLattice::Init(CellLattice * subject) {
	// just copy the matrix from subject over to this lattice
	n_cols = subject->nCols();
	n_rows = subject->nRows();

	matrix = new long[n_rows*n_cols];

	for(int i=0;i<n_rows;i++) {
		for(int j=0;j<n_cols;j++) {
			matrix[i*n_cols+j] = subject->operator [](i)[j];
		}
	}

	isClone = true;
}

void CellLattice::FreeMem() {
	// delete all the cells

	if(!isClone)
	for(int i=0;i<n_rows*n_cols;i++) {
		if(matrix[i]!=NULL) {
			delete ((Cell *) matrix[i]);
		}
	}

	// delete the matrix itself
	if(matrix!=NULL) delete matrix;

}

CellLattice::~CellLattice() {

}

int CellLattice::nCols() { return n_cols; }
int CellLattice::nRows() { return n_rows; }

long * CellLattice::operator [](int row) {
  if(row<n_rows && matrix!=NULL)
    return &(matrix[row*n_cols]);
  else
    return NULL;
}

void CellLattice::AddCell(Cell * newCell) {
	for(int i=0;i<n_rows;i++) {
		for(int j=0;j<n_cols;j++) {
			if(matrix[i*n_cols+j]==NULL) {
				// found a spot to add
				matrix[i*n_cols+j] = (long) newCell;
				return;
			}
		}
	}

	// otherwise out of room, so do something bad
	fprintf(stderr,"Out of room in Cell Lattice\n");
	int a = 0;
	a = 10 / a;
}

void CellLattice::dump() {
	Cell * current;
	for(int i=0;i<n_rows;i++) {
		for(int j=0;j<n_cols;j++) {
			current = (Cell *) matrix[i*n_cols+j];
			if(current==NULL) {
				printf("<  >");
			} else {
				printf("<%i",current->myNumber);
				if(current->myNumber<10) printf(" ");
				printf(">");
			}
		}
		printf("\n");
	}
}

