#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#include "SA_twolf.h"
#include "CellLattice.h"

int main(int argc, char * argv[]) {

	struct timeb starttime; // will hold algorithm start time
	struct timeb stoptime;  // will hold algorithm stop time


	if(argc<=1||argc>5) {
		printf("\nUsage is:\n");
		printf("timberwolf <filename> [T0] [M] [Beta]\n");
		printf("<filename> - Name of input file\n");
		printf("[T0]       - Initial temperature = 65536.0\n");
		printf("[M]        - Initial Metropolis Effort = 10\n");
		printf("[Beta]     - Metropolis effort increase parameter (>1) = 1.05\n");
		return -1;
	}
	
	double T0 = 65536.0;
	int M = 10;
	double beta = 1.05;

	switch(argc) {
	case 5:
		beta = strtod(argv[4],NULL);
	case 4:
		M = atoi(argv[3]);
	case 3:
		T0 = strtod(argv[2],NULL);
	default:
		break;
	}

	ftime( &starttime );
	SimulatedAnnealing(argv[1],T0,M,beta);
	ftime( &stoptime );

	int seconds = stoptime.time - starttime.time;
	int milliseconds = stoptime.millitm - starttime.millitm;
	if(milliseconds<0) {
		milliseconds+= 1000;
		seconds--;
	}

	// print running time

	fprintf(stdout, "\nRunning time of TWolf: %i seconds\t%i milliseconds.\n",
            seconds, milliseconds );

	return 0;
}

